collisionResolution = filteredSystem({ collisionBetween = T.Collision }, function(e, dt, system)
    local collidedInto, collider = e.collisionBetween[1], e.collisionBetween[2]
    local colliderTop = collidedInto.position.y

    if collidedInto.isSolid then
        -- Assumes impact from the top
        collider.position.y = colliderTop - collider.size.y
    end

    if collidedInto.canBounce and collider.canBeBounced then
        collider.velocity.x = collider.velocity.x + collidedInto.canBounce.flat.x + collider.canBeBounced.flat.x

        collider.velocity.x = collider.velocity.x * collidedInto.canBounce.mult.x * collider.canBeBounced.mult.x

        -- abs() makes sure we always push upward
        collider.velocity.y = math.abs(collider.velocity.y)
            + collidedInto.canBounce.flat.y
            + collider.canBeBounced.flat.y

        collider.velocity.y = collider.velocity.y * collidedInto.canBounce.mult.y * collider.canBeBounced.mult.y
    end

    if collider.focusOnCollide then
        system.world:addEntity({
            focusPriority = collider.focusOnCollide,
            position = {
                x = collider.position.x,
                y = collider.position.y,
            }
        })
    end

    if collider.stopMovingOnCollision then
        collider.velocity = nil
        collider.canCollideWith = nil
        system.world:addEntity(collider)
    end

    if collider.onCollidingRemove then
        for _, key in ipairs(collider.onCollidingRemove) do
            collider[key] = nil
        end
        collider.onCollidingRemove = nil
    end

    if collidedInto.score then
        Score:add(collidedInto.score)
    end

    if collidedInto.expireAfterCollision then
        system.world:removeEntity(collidedInto)
    end

    if collidedInto.collectable then
        system.world:addEntity({ collected = collidedInto.collectable })
    end

    -- if collidedInto.sparkOnCollision and collider.sparkOnCollision and (math.abs(collider.velocity.x) + math.abs(collider.velocity.y)) > 2 then
        -- local size = { x = 1, y = 1 }
        -- local spark = {
            -- position = {
                -- x = collider.position.x,
                -- y = collider.position.y + collider.size.y - 2,
            -- },
            -- velocity = {
                -- x = -collider.velocity.x,
                -- y = 150,
            -- },
            -- size = size,
            -- canCollideWith = 1,
            -- mass = 1,
            -- drawAsSprite = Spark,
            -- expireAfterCollision = true,
        -- }
        -- system.world:addEntity(spark)
    -- end

    system.world:removeEntity(e)
end)
